<?php
/*
 * (c) Kiva Corp.
 */

namespace Plugin\Proteger\Service\PurchaseFlow\Processor;

use Eccube\Annotation\OrderFlow;
use Eccube\Annotation\ShoppingFlow;
use Eccube\Entity\ItemHolderInterface;
use Eccube\Entity\Order;
use Eccube\Service\PurchaseFlow\InvalidItemException;
use Eccube\Service\PurchaseFlow\ItemHolderValidator;
use Eccube\Service\PurchaseFlow\PurchaseContext;

/**
 * Proteger不正購入判定
 *
 * @ShoppingFlow
 * @OrderFlow
 */
class ProtegerFraudValidator extends ItemHolderValidator
{
   /**
    * @param ItemHolderInterface $itemHolder
    * @param PurchaseContext $context
    *
    * @throws InvalidItemException
    */
    public function validate(ItemHolderInterface $itemHolder, PurchaseContext $context)
    {
        if (!$itemHolder instanceof Order) {
	    return;
	}

	$isProteger = false;
	$protegerItems = array();
	foreach ($itemHolder->getOrderItems() as $item) {
            if (!$item->isProduct()) {
		continue;
	    }

	    if (strpos($item->getProductCode(), 'proteger') !== false) {
		array_push($protegerItems, $item);

		$isProteger = true;
	    }
	}

	if (!$isProteger) {
	    return;
	}

	if ($this->existParentProduct($itemHolder, $protegerItems)) {
	    throw new InvalidItemException('保証プランに紐づく商品がカートに入っていません。');
	} elseif ($this->planQuantity($itemHolder, $protegerItems)) {
		throw new InvalidItemException('保証プランの個数がそれに紐づく商品の個数を上回ってます。');
	} elseif ($this->multipleKindPlan($itemHolder, $protegerItems)) {
	    throw new InvalidItemException('商品に対し保証プランの種類が複数あります。');
	} else {
	    return;
	}
    }

    /**
     * 保証プランに紐づく商品がカートに入っているか確認する。
     *
     * @param ItemHolderInterface $itemHolder $protegerItems
     */
    protected function existParentProduct(ItemHolderInterface $itemHolder, $protegerItems) {
        foreach ($protegerItems as $item) {
	    $planName = $item->getProductName();
	    $array = explode('-', $planName);
	    $parentProductClassId = end($array);

	    $haveParentProduct = false;
	    foreach ($itemHolder->getOrderItems() as $item) {
                if (!$item->isProduct()) {
		    continue;
	        }

	        if ($item->getProductClass()->getId() == $parentProductClassId) {
		    $haveParentProduct = true;
	        }
	    }

	    if (!$haveParentProduct) {
		return true;
	    }
	}

	return false;
    }

    /**
     * 保証プランの個数がそれに紐づく商品の個数を上回ってないか確認する。
     *
     * @param ItemHolderInterface $itemHolder $protegerItems
     */
    protected function planQuantity(ItemHolderInterface $itemHolder, $protegerItems) {
	foreach ($protegerItems as $protegerItem) {
	    $protegerQuantity = $protegerItem->getQuantity();
	    $protegerProductName = $protegerItem->getProductName();

	    foreach ($itemHolder->getOrderItems() as $item) {
                if (!$item->isProduct()) {
                    return;
                }

                $productClassId = $item->getProductClass()->getId();
                $keyArray = explode('-', $protegerProductName);
                $id = end($keyArray);

                if (isset($id) && $id == $productClassId) {
                    if ($item->getQuantity() < $protegerQuantity) {
                        return true;
                    }
	        }
	    }
	}

	return false;
    }

    /**
     * 商品に対し保証プランの種類が複数ないか確認する。
     *
     * @param ItemHolderInterface $itemHolder $protegerItems
     */
    protected function multipleKindPlan(ItemHolderInterface $itemHolder, $protegerItems) {
	$plans = array();
	foreach ($protegerItems as $item) {
	    $planName = $item->getProductName();
	    $array = explode('-', $planName);
	    $parentProductClassId = end($array);

	    if (!isset($plans[$parentProductClassId])) {
		$plans[$parentProductClassId] = array();
	    }

	    if (!in_array($planName, $plans[$parentProductClassId])) {
		array_push($plans[$parentProductClassId], $planName);
	    }
	}

	if (count($plans) > 0) {
	    foreach ($plans as $key => $plan) {
	        if (count($plan) > 1) {
		    return true;
	        }
	    }
	}

	return false;
    }
}
